﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using System;
using System.Diagnostics;
using System.Security.Cryptography;
using System.Text;

namespace SSODemo1.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;
        private readonly IConfiguration Configuration;
        private readonly IUserService userService;

        public HomeController(ILogger<HomeController> logger, IConfiguration configuration, IUserService userService)
        {
            Configuration = configuration;
            this.userService = userService;
            _logger = logger;
        }

        /// <summary>
        /// 登录页
        /// </summary>
        /// <param name="bbs_redirectUrl"></param>
        /// <returns></returns>
        public IActionResult Index(string bbs_redirectUrl)
        {
            var user = userService.GetUser();

            LoginModel loginModel = new ()
            {
                UserName = user.UserName,
                RedirectUrl = bbs_redirectUrl,
            };

            return View(loginModel);
        }


        [HttpPost]
        public IActionResult Login(LoginModel loginModel)
        {
            var user = userService.GetUser();

            LoginData loginData = GetLoginData(user);

            string redirectUrl = GetRedirectUrl(loginData, loginModel.RedirectUrl);

            return Redirect(redirectUrl);
        }

        /// <summary>
        /// 登录信息
        /// </summary>
        /// <param name="user"></param>
        /// <returns></returns>
        private LoginData GetLoginData(User user)
        {
            string sign = "";

            StringBuilder sb = new ();

            if (!string.IsNullOrWhiteSpace(user.Avatar))
                sb.Append(user.Avatar);
            if (!string.IsNullOrWhiteSpace(user.Email))
                sb.Append(user.Email);
            if (!string.IsNullOrWhiteSpace(user.Mobile))
                sb.Append(user.Mobile);

            sb.Append(user.OpenId);
            var time = (DateTime.Now.ToUniversalTime().Ticks - 621355968000000000) / 10000000;
            sb.Append(time);

            if (!string.IsNullOrWhiteSpace(user.Avatar))
                sb.Append(user.Avatar);

            if (!string.IsNullOrWhiteSpace(user.UserName))
                sb.Append(user.UserName);

            sb.Append(Configuration.GetValue<string>("SSOLoginKey"));

            using (var md5 = MD5.Create())
            {
                string sbResult = sb.ToString();
                var result = md5.ComputeHash(Encoding.UTF8.GetBytes(sbResult));

                sign = BitConverter.ToString(result).Replace("-", "").ToLower();
            }
            return new (user.OpenId, user.UserName, user.Mobile, user.Email, user.Avatar, time, sign);
        }

        private static string GetRedirectUrl(LoginData loginData, string redirectUrl)
        {
            return $"{redirectUrl}&openId={loginData.openId}&username={loginData.username}&mobile={loginData.mobile}&email={loginData.email}&avatar={loginData.avatar}&time={loginData.time}&sign={loginData.sign}";
        }

        /// <summary>
        /// 登出
        /// </summary>
        /// <param name="bbs_redirectUrl"></param>
        /// <returns></returns>
        public IActionResult Logout(string bbs_redirectUrl)
        {
            return Redirect(bbs_redirectUrl);
        }

        #region SystemError

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }

        #endregion


    }

}
